#include "../include/Crypt.h"

Crypt::Crypt(){

}

/** simple
 @brief Simple Crypt replacement encrypted. It will replace each character from a sentances with the character at a defined distance in alphabetical.
 @see replaceChar
 @parameter string sentances, the sentance to encrypted
 @parameter key, the distance to shift
 @return string, the encrypted sentance
 **/
std::string Crypt::cesar( std::string sentances, int key, char start, char end ){
  std::string result = "";

  for( char character : sentances ){
    result += Crypt::isLetter( character, start, end )?
      Crypt::replaceChar( character, key, start, end ) :
	character;
  }
  
  return result;
}

/** vigenere
 @brief A lot mof stronger than the previous, even it's still a weak encryption algorythm. This is the same principle, however the key is modified for each character. The most elegant way to defined this dequance is with another sentance. Each character of the key sentance will defined the "a", or the key if cast in an integer
 @see loopInString
 @see replaceChar
 @see alphabet
 @return string, the vigenere encrypted sentance
 **/
std::string Crypt::vigenere( std::string sentance, std::string key, char start, char end ){
  std::string result = "";
  int k = 0;

  const char* c_sentance = sentance.c_str();

  for( int count = 0 ;  count < (int)sentance.length(); count++ ){
    char character = c_sentance[ count ] ;
    k = alphabetIndex( loopInString( key, count+1 ), start, end);

    if( k == 0 ){
      result += character;
    }else{
      result += Crypt::replaceChar( character, k, start, end ) ;
    }
  }

  return result;
}

/**
 **/
std::string Crypt::atbash( std::string sentance, char start, char end ){
  std::string result = "";
  int k = 0;
  int max = (int)end ;

  for( char character : sentance ){
    if( Crypt::isLetter( character, start, end ) ){
      k = Crypt::alphabetIndex( character, start, end ) ;
      result += (char)( max - k + 1) ;
    }else{
      result += character;
    }
  }

  return result;
}

/** replaceChar
 @brief replace a character with the next character at key distance in the alphabetical order, and loop at z.
 @parameter char ch, the character to replace
 @parameter int key, the distance to look at
 @return string, the replacement character
 **/
char Crypt::replaceChar( char ch, int key, char start, char end ){
  ch = (char)std::tolower( ch );
  int max = (int)end ;

  if( !Crypt::isLetter( ch, start, end ) ){
    return ch;
  }

  if( ( (int)ch + key) > max ){
    key -= (int)end - (int)start + 1;
  }

  return (char)((int)ch + key);
}

/** loopInString
 @brief return a character of the sentance, defined by the integer and the character place in the sentance. However, if the integer is longer than the sentance, it will loop back at the beginning.
 @return char
 **/
char Crypt::loopInString( std::string str, int key ){
  
  key -= ((int)str.length() < key )? str.length() : 0 ;

  return (Crypt::isLetter( str.c_str()[ key - 1 ] ))? str.c_str()[ key - 1 ] : 0 ;
}

/** alphabetIndex
 @brief return the number of the character in the alphabet order. Capital letter are lowered, other character are return untouched.
 @paramater char ch, the character to look at
 @return int, ordered index in the alphabet
 **/
int Crypt::alphabetIndex( char ch, char start, char end ){
  ch = (char)std::tolower( ch );
  int min = (int)start;

  if( Crypt::isLetter( ch, start, end ) ){
    return int(ch) - min + 1;
  }
  
  return ch;
}

/** isLetter
 @brief Return true is the character is a letter, which is the range between start and end
 **/
bool Crypt::isLetter( char ch, char start, char end ){
  ch = (char)std::tolower( ch );
  
  if(( (int)ch < (int)start ) || ( (int)ch > (int)end )){
    return false;
  }
  return true;
}
